// Soviet Union 2011 sound engine example program

// Matej Horvat (HorvatM)
// February 25, 2011

// Web: http://matejhorvat.si/
// Electronic mail: matej.horvat@guest.arnes.si

// This example program shows how to use the Soviet Union 2011 sound engine.
// Controls: A/B/L/R: play sounds, Select: stop sounds, Start: pause sounds
//
// The sound engine works by keeping track of sounds that are playing. After
// each video frame, it checks each sound to see whether it should switch to
// a different note, stop the sound, or start playing it from the beginning.
//
// The following functions are provided:
//
// * InitSound
//   This must be called at the start of the game to initialize the VSU for
//   use with the sound engine.
//
// * PlaySound
//   Schedules a sound to play.
//
// * DoSounds
//   This must be called after each video frame for actually playing sounds.
//
// * StopSound
//   Stops the sound that's playing at the specified channel. This is not
//   normally useful to call directly, unless, for example, you start playing
//   a music track at the start of a game level, forcing it to occupy the
//   WAVE1 channel, then change the music track in the middle of the game by
//   calling StopSound(WAVE1) and playing a different music track.
//
// * StopAllSounds
//   Makes silence and frees all sound channels.
//
// * PauseAllSounds
//   Makes silence, but doesn't free any sound channels. Playback will be
//   resumed the next time DoSounds is called.
//
// The engine is very basic, but can be expanded. Some ideas:
//
// * Add panning for 3D sound.
//
// * Support a different waveform for each sound.
//
// * Change the sound format to support variable-length notes.
//
// * Add support for the sweep and noise channels.
//
// * Have PlaySound return the index of the sound channel of the new sound,
//   then pass that to StopSound for more control over playback, or modify
//   the FramesPerNote property to change the speed of the sound.
//
// You may use the sound engine in your projects as long as you properly give
// credit.

#include "libgccvb/libgccvb.h"  // gccVB library
#include "player/notes.h"     // Note constants by DanB
#include "su11sfx.h"   // Sound engine header
#include "su11sfx.c"   // Sound engine implementation
#include "sounddat.h"  // Actual sounds

#include "player/midiPlayer.h"

int main ()
{
  u16 Buttons;                            // Button cache.
  unsigned int FramesUntilNextSound = 0;  // For demonstration purposes only.

  // Turn on the display because we need the VIP to synchronize sounds.
  vbDisplayOn();

  // Set up the VSU.
  InitSound();
  
  //inti MidiLibrary
  ml_iniLibrary();
	
  //init MidiPlayer
  mp_init();

  for(;;)
  {
    // Wait until end of frame.
    while(!(VIP_REGS[INTPND] & XPEND));
    VIP_REGS[INTCLR] = XPEND;
    
    // Take care of playing sounds.
    DoSounds();
	 mp_playMusic();
    
    // Schedule new sounds to play.
    if(!FramesUntilNextSound)
    {
      Buttons = vbReadPad();
      if(Buttons & K_BTNS)
      {
        if(Buttons & K_LT) PlaySound(sndAmmoPickup);
        if(Buttons & K_RT) PlaySound(sndPickupPickup);
        if(Buttons & K_A)  PlaySound(sndCannonShot);
        if(Buttons & K_B)  
		{
			mp_playerStop(); 	// stop player
			mp_loadSong(0);		// load song
			mp_playerStart();	// start loaded song
		}
      
        FramesUntilNextSound = 25;  // So we don't make a flood of sounds.
      }
    }
    else
    {
      FramesUntilNextSound--;
    }
    
    // Pausing and stopping.
    if(vbReadPad() & K_STA)
    {
      PauseAllSounds();
      while(vbReadPad() & K_STA);
    }
    if(vbReadPad() & K_SEL) StopAllSounds();
  }
}
